//
// Copyright (c) 2016, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Xml;

using MindFusion.Scheduling;
using ImageAlignment = MindFusion.Drawing.ImageAlignment;


namespace MindFusion.Scheduling.WinForms.Samples.CS.AdvancedItemDrawing
{
	/// <summary>
	/// Represents an ordinary appointment with a round shape.
	/// </summary>
	[ItemViewer(typeof(CustomAppointmentViewer))]
	[ItemEditor(typeof(ItemEditorEx))]
	public class CustomAppointment : Appointment
	{
		/// <summary>
		/// Initializes a new instance of the RoundAppointment class.
		/// </summary>
		public CustomAppointment()
		{
			radius = 6;
			transparency = 100;
			headerImages = new List<Image>();
			contentsImage = null;
			contentsImageAlignment = ImageAlignment.Left | ImageAlignment.Top;
		}

		/// <summary>
		/// Creates an exact shallow clone of this appointment.
		/// </summary>
		public override object Clone()
		{
			CustomAppointment clone = new CustomAppointment();

			clone.AllDayEvent = this.AllDayEvent;
			clone.AllowChangeEnd = this.AllowChangeEnd;
			clone.AllowChangeStart = this.AllowChangeStart;
			clone.AllowMove = this.AllowMove;
			clone.DescriptionText = this.DescriptionText;
			clone.EndTime = this.EndTime;
			clone.HeaderText = this.HeaderText;
			clone.Location = this.Location;
			clone.Locked = this.Locked;
#if !PocketPC
			clone.PointedSelectedStyle = this.PointedSelectedStyle.Clone() as Style;
			clone.PointedStyle = this.PointedStyle.Clone() as Style;
#endif
			clone.Priority = this.Priority;
			clone.Reminder = this.Reminder;
			clone.SelectedStyle = this.SelectedStyle.Clone() as Style;
			clone.StartTime = this.StartTime;
			clone.Style = this.Style.Clone() as Style;
			clone.Tag = this.Tag;
			clone.Task = this.Task;
			clone.Visible = this.Visible;

			foreach (Resource resource in this.Resources)
				clone.Resources.Add(resource);

			foreach (Contact contact in this.Contacts)
				clone.Contacts.Add(contact);

			clone.Radius = this.Radius;
			clone.Transparency = this.Transparency;

			foreach (Image headerImage in this.HeaderImages)
				clone.HeaderImages.Add(headerImage);

			clone.ContentsImage = this.ContentsImage;
			clone.ContentsImageAlignment = this.ContentsImageAlignment;

			return clone;
		}

		/// <summary>
		/// Appointment.CopyOccurrence override.
		/// </summary>
		protected override void CopyOccurrence(Item master)
		{
			this.radius = (master as CustomAppointment).radius;
			this.transparency = (master as CustomAppointment).transparency;
			this.headerImages = new List<Image>();
			this.headerImages.AddRange((master as CustomAppointment).headerImages);
			this.contentsImage = (master as CustomAppointment).contentsImage;
			this.contentsImageAlignment = (master as CustomAppointment).contentsImageAlignment;

			base.CopyOccurrence(master);
		}

		/// <summary>
		/// Appointment.SaveTo override.
		/// </summary>
		public override void SaveTo(BinaryWriter writer, BinarySerializationContext context)
		{
			base.SaveTo(writer, context);

			context.WriteInt(radius, writer);
			context.WriteInt(transparency, writer);

			context.WriteInt(headerImages.Count, writer);
			foreach (Image headerImage in headerImages)
				context.WriteImage(headerImage, writer);

			context.WriteImage(contentsImage, writer);
			context.WriteEnum(contentsImageAlignment, writer);
		}

		/// <summary>
		/// Appointment.LoadFrom override.
		/// </summary>
		public override void LoadFrom(BinaryReader reader, BinarySerializationContext context)
		{
			base.LoadFrom(reader, context);

			radius = context.ReadInt(reader);
			transparency = context.ReadInt(reader);

			int count = context.ReadInt(reader);
			headerImages.Clear();
			for (int i = 0; i < count; i++)
				headerImages.Add(context.ReadImage(reader));

			contentsImage = context.ReadImage(reader);
			contentsImageAlignment = (ImageAlignment)context.ReadEnum(reader);
		}

		/// <summary>
		/// Appointment.SaveTo override.
		/// </summary>
		public override void SaveTo(XmlElement element, XmlSerializationContext context)
		{
			base.SaveTo(element, context);

			context.WriteInt(radius, "radius", element);
			context.WriteInt(transparency, "transparency", element);

			XmlElement headerImagesElement = context.AddChildElement("headerImages", element);
			foreach (Image headerImage in headerImages)
			{
				XmlElement imageElement = context.AddChildElement("image", headerImagesElement);
				context.WriteImage(headerImage, "value", imageElement);
			}

			context.WriteImage(contentsImage, "contentsImage", element);
			context.WriteEnum(contentsImageAlignment, "contentsImageAlignment", element);
		}

		/// <summary>
		/// Appointment.LoadFrom override.
		/// </summary>
		public override void LoadFrom(XmlElement element, XmlSerializationContext context)
		{
			base.LoadFrom(element, context);

			radius = context.ReadInt("radius", element);
			transparency = context.ReadInt("transparency", element);

			XmlElement headerImagesElement = element.SelectSingleNode("headerImages") as XmlElement;
			headerImages.Clear();
			if (headerImagesElement != null)
			{
				foreach (XmlElement headerImageElement in headerImagesElement.SelectNodes("image"))
					headerImages.Add(context.ReadImage("value", headerImageElement));
			}

			contentsImage = context.ReadImage("contentsImage", element);
			contentsImageAlignment = (ImageAlignment)context.ReadEnum("contentsImageAlignment", element);
		}


		/// <summary>
		/// Gets or sets the round rect radius of this appointment.
		/// </summary>
		public int Radius
		{
			get { return radius; }
			set
			{
				if (radius == value)
					return;

				radius = value;

				// Invalidate item visuals
				OnVisualsChanged();

				// Changing the selected style causes
				// recurrent appointments to become exceptions
				if (Recurrence != null)
					Recurrence.MarkException(this, false);
			}
		}

		/// <summary>
		/// Gets or sets the overal transparency of this appointment.
		/// </summary>
		public int Transparency
		{
			get { return transparency; }
			set
			{
				if (transparency == value)
					return;

				transparency = value;

				// Invalidate item visuals
				OnVisualsChanged();

				// Changing the selected style causes
				// recurrent appointments to become exceptions
				if (Recurrence != null)
					Recurrence.MarkException(this, false);
			}
		}

		/// <summary>
		/// Gets or sets the custom image displayed in the item's header.
		/// </summary>
		public List<Image> HeaderImages
		{
			get { return headerImages; }
		}

		/// <summary>
		/// Gets or sets the custom image displayed in the item's contents.
		/// </summary>
		public Image ContentsImage
		{
			get { return contentsImage; }
			set
			{
				if (contentsImage == value)
					return;

				contentsImage = value;

				// Invalidate item visuals
				OnVisualsChanged();

				// Changing the selected style causes
				// recurrent appointments to become exceptions
				if (Recurrence != null)
					Recurrence.MarkException(this, false);
			}
		}

		/// <summary>
		/// Gets or sets the alignment of the image displayed in the
		/// item's contents relative to the contents bounding rectangle.
		/// </summary>
		public ImageAlignment ContentsImageAlignment
		{
			get { return contentsImageAlignment; }
			set
			{
				if (contentsImageAlignment == value)
					return;

				contentsImageAlignment = value;

				// Invalidate item visuals
				OnVisualsChanged();

				// Changing the selected style causes
				// recurrent appointments to become exceptions
				if (Recurrence != null)
					Recurrence.MarkException(this, false);
			}
		}

		public void UpdateVisuals()
		{
			OnVisualsChanged();
		}


		private int radius;
		private int transparency;
		private List<Image> headerImages;
		private Image contentsImage;
		private ImageAlignment contentsImageAlignment;
	}
}
