//
// Copyright (c) 2016, MindFusion LLC - Bulgaria.
//

using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;

using MindFusion.Scheduling;
using ImageAlignment = MindFusion.Drawing.ImageAlignment;


namespace MindFusion.Scheduling.WinForms.Samples.CS.AdvancedItemDrawing
{
	/// <summary>
	/// An appointment viewer with a round rectangle shape.
	/// </summary>
	public class CustomAppointmentViewer : IItemViewer
	{
		/// <summary>
		/// Initializes a new instance of the RoundAppointmentViewer class.
		/// </summary>
		public CustomAppointmentViewer()
		{
		}

		/// <summary>
		/// IItemViewer.Draw override.
		/// </summary>
		public void Draw(ItemDrawContext context)
		{
			if (!(context.Graphics is MindFusion.Drawing.GdiGraphics))
				return;

			MindFusion.Drawing.GdiGraphics graphics =
				context.Graphics as MindFusion.Drawing.GdiGraphics;

			// Create an off-screen bitmap to render the appointment to.
			// This will enable us to apply overal transparency at the end
			Bitmap obitmap = new Bitmap(context.Bounds.Width, context.Bounds.Height, graphics.Graphics);
			Graphics ographics = Graphics.FromImage(obitmap);
			ographics.TranslateTransform(-context.Bounds.X, -context.Bounds.Y);
			MindFusion.Drawing.IGraphics trueGraphics = context.Graphics;
			context.Graphics = new MindFusion.Drawing.GdiGraphics(ographics);

			CustomAppointment appointment = context.Item as CustomAppointment;
			int radius = appointment.Radius;

			// Calculate visible corners
			bool leftTop = true;
			bool rightTop = true;
			bool leftBottom = true;
			bool rightBottom = true;
			if (context.IsHorizontal)
			{
				if (!context.StartsHere)
				{
					leftTop = false;
					leftBottom = false;
				}

				if (!context.EndsHere)
				{
					rightTop = false;
					rightBottom = false;
				}
			}
			else
			{
				if (!context.StartsHere)
				{
					leftTop = false;
					rightTop = false;
				}

				if (!context.EndsHere)
				{
					leftBottom = false;
					rightBottom = false;
				}
			}

			// Get some frequently used properties
			MindFusion.Drawing.IGraphics g = context.Graphics;
			Rectangle bounds = context.Bounds;
			bounds.Width -= 1;
			bounds.Height -= 1;

			// Save graphics state
			GraphicsState state = g.Save();

			// Enable smoothing
			g.SmoothingMode = SmoothingMode.AntiAlias;
			g.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;

			// Create a round rectangle GraphicsPath for the item
			GraphicsPath path = CreateRoundRect(
				bounds.X, bounds.Y, bounds.Width, bounds.Height, radius,
				leftTop, rightTop, leftBottom, rightBottom);

			// Set the path for clipping
			Region oldClip = g.Clip;
			Region newClip = new Region(path);
			if (oldClip != null)
				newClip.Intersect(oldClip);
			g.Clip = newClip;

			// Fill header
			int headerSize = context.Calendar.ItemSettings.HeaderSize;
			Rectangle r;
			if (headerSize > 0)
			{
				Rectangle headerBounds = bounds;
				headerBounds.Height = headerSize;

				// Set new clip
				Region oldClip2 = g.Clip;
				Region newClip2 = new Region(headerBounds);
				if (oldClip2 != null)
					newClip2.Intersect(oldClip2);
				g.Clip = newClip2;

				Brush headerBrush = context.Style.HeaderBrush.CreateGdiBrush(headerBounds);
				g.FillRectangle(headerBrush, headerBounds);
				headerBrush.Dispose();

				// Draw the 'doesn't start here' icon
				r = headerBounds;
				if (!context.StartsHere)
				{
					Point[] poly = new Point[]
					{
						new Point(r.Left + 4, r.Y + r.Height / 2),
						new	Point(r.Left + 8, r.Y + r.Height / 2 - 4),
						new	Point(r.Left + 8, r.Y + r.Height / 2 + 4)
					};

					g.FillPolygon(Brushes.Black, poly);
					headerBounds.X += 6;
					headerBounds.Width -= 6;
				}

				// Draw the 'doesn't end here' icon
				r = headerBounds;
				if (!context.EndsHere)
				{
					Point[] poly = new Point[]
					{
						new Point(r.Right - 4, r.Y + r.Height / 2),
						new	Point(r.Right - 8, r.Y + r.Height / 2 - 4),
						new	Point(r.Right - 8, r.Y + r.Height / 2 + 4)
					};

					g.FillPolygon(Brushes.Black, poly);
					headerBounds.Width -= 8;
				}

				// Draw the item icons in its header
				int iconSize = 16;
				if (context.Item.RecurrenceState == RecurrenceState.Exception ||
					context.Item.RecurrenceState == RecurrenceState.Occurrence)
				{
					r = headerBounds;
					r.Inflate(-2, -2);
					r.Width = iconSize;
					r.Y = r.Y + r.Height / 2 - iconSize / 2;
					r.Height = iconSize;

					context.DrawRecurrenceIcon(ref r, false);

					headerBounds.X += iconSize + 2;
					headerBounds.Width -= iconSize + 2;
				}

				if (context.Item.Reminder != null)
				{
					r = headerBounds;
					r.Inflate(-2, -2);
					r.Width = iconSize;
					r.Y = r.Y + r.Height / 2 - iconSize / 2;
					r.Height = iconSize;

					context.DrawReminderIcon(ref r, false);

					headerBounds.X += iconSize + 2;
					headerBounds.Width -= iconSize + 2;
				}

				// Draw a small label representing the start time of the item
				string label = appointment.StartTime.ToString("HH:mm");
				SizeF size = g.MeasureString(label, context.Style.Font);
				r = headerBounds;
				r.Inflate(-2, -2);
				r.X = r.Right - (int)size.Width;
				r.Width = (int)size.Width;
				r.Y = r.Y + r.Height / 2 - (int)size.Height / 2;
				r.Height = (int)size.Height;
				g.FillRectangle(Brushes.White, r);
				Pen labelFramePen = new Pen(context.Style.LineColor, 0);
				g.DrawRectangle(labelFramePen, r);
				labelFramePen.Dispose();
				g.DrawString(label, context.Style.Font, Brushes.Gray, r.X, r.Y);
				headerBounds.Width -= (int)size.Width + 2;

				// Draw the clock icon in the right corner
				r = headerBounds;
				r.Inflate(-2, -2);
				r.X = r.Right - iconSize;
				r.Width = iconSize;
				r.Y = r.Y + r.Height / 2 - iconSize / 2;
				r.Height = iconSize;
				context.DrawStartClock(ref r, false);
				headerBounds.Width -= iconSize + 2;

				foreach (Image headerImage in appointment.HeaderImages)
				{
					r = headerBounds;
					r.Inflate(-2, -2);
					r.Width = headerImage.Width;
					r.Y = r.Y + r.Height / 2 - headerImage.Height / 2;
					r.Height = headerImage.Height;
					r.Offset(2, 2);

					g.DrawImage(headerImage, r.Location);

					headerBounds.X += headerImage.Width + 2;
					headerBounds.Width -= headerImage.Width + 2;
				}

				// Draw header text
				r = headerBounds;
				context.DrawHeaderText(ref r, false, false);

				// Restore clip
				g.Clip = oldClip2;
				newClip2.Dispose();
			}

			// Fill conents
			Rectangle contentsBounds = bounds;
			contentsBounds.Y += headerSize;
			contentsBounds.Height -= headerSize;

			if (contentsBounds.Width > 0 && contentsBounds.Height > 0)
			{
				Brush contentsBrush = context.Style.Brush.CreateGdiBrush(
					new RectangleF(contentsBounds.X, contentsBounds.Y, contentsBounds.Width, contentsBounds.Height));
				g.FillRectangle(contentsBrush, contentsBounds);
				contentsBrush.Dispose();

				// Get available space for contents
				r = contentsBounds;
				r.X += context.Style.TextLeftMargin;
				r.Width -= (context.Style.TextLeftMargin + context.Style.TextRightMargin);
				r.Y += context.Style.TextTopMargin;
				r.Height -= (context.Style.TextTopMargin + context.Style.TextBottomMargin);

				// Set new clip
				Region oldClip2 = g.Clip;
				Region newClip2 = new Region(r);
				if (oldClip2 != null)
					newClip2.Intersect(oldClip2);
				g.Clip = newClip2;

				// Draw the contents image
				PointF[] contentsPoly = null;
				if (appointment.ContentsImage != null &&
					appointment.ContentsImageAlignment != ImageAlignment.Unspecified)
				{
					ImageAlignment a = appointment.ContentsImageAlignment;
					context.DrawImage(appointment.ContentsImage, r, a);

					int w = appointment.ContentsImage.Width;
					if (w > r.Width)
						w = r.Width;

					int h = appointment.ContentsImage.Height;
					if (h > r.Height)
						h = r.Height;

					// Calculate the contents poly depending on the image alignment
					if (a == (ImageAlignment.Left | ImageAlignment.Top))
					{
						// Left-top
						contentsPoly = new PointF[]
						{
							new PointF(r.Left + w, r.Top),
							new	PointF(r.Right, r.Top),
							new PointF(r.Right, r.Bottom),
							new PointF(r.Left, r.Bottom),
							new PointF(r.Left, r.Top + h),
							new PointF(r.Left + w, r.Top + h)
						};
					}
					else if (a == ImageAlignment.Top)
					{
						// Top
						contentsPoly = new PointF[]
						{
							new PointF(r.Left, r.Top),
							new PointF(r.Left + r.Width / 2 - w / 2, r.Top),
							new PointF(r.Left + r.Width / 2 - w / 2, r.Top + h),
							new	PointF(r.Left + r.Width / 2 + w / 2, r.Top + h),
							new PointF(r.Left + r.Width / 2 + w / 2, r.Top),
							new	PointF(r.Right, r.Top),
							new PointF(r.Right, r.Bottom),
							new PointF(r.Left, r.Bottom)
						};
					}
					else if (a == (ImageAlignment.Right | ImageAlignment.Top))
					{
						// Right-top
						contentsPoly = new PointF[]
						{
							new PointF(r.Left, r.Top),
							new	PointF(r.Right - w, r.Top),
							new PointF(r.Right - w, r.Top + h),
							new PointF(r.Right, r.Top + h),
							new PointF(r.Right, r.Bottom),
							new PointF(r.Left, r.Bottom)
						};
					}
					else if (a == ImageAlignment.Right)
					{
						// Right
						contentsPoly = new PointF[]
						{
							new PointF(r.Left, r.Top),
							new PointF(r.Right, r.Top),
							new PointF(r.Right, r.Top + r.Height / 2 - h / 2),
							new PointF(r.Right - w, r.Top + r.Height / 2 - h / 2),
							new PointF(r.Right - w, r.Top + r.Height / 2 + h / 2),
							new PointF(r.Right, r.Top + r.Height / 2 + h / 2),
							new PointF(r.Right, r.Bottom),
							new PointF(r.Left, r.Bottom)
						};
					}
					else if (a == (ImageAlignment.Right | ImageAlignment.Bottom))
					{
						// Right-bottom
						contentsPoly = new PointF[]
						{
							new PointF(r.Left, r.Top),
							new PointF(r.Right, r.Top),
							new PointF(r.Right, r.Bottom - h),
							new PointF(r.Right - w, r.Bottom - h),
							new PointF(r.Right - w, r.Bottom),
							new PointF(r.Left, r.Bottom)
						};
					}
					else if (a == ImageAlignment.Bottom)
					{
						// Bottom
						contentsPoly = new PointF[]
						{
							new PointF(r.Left, r.Top),
							new	PointF(r.Right, r.Top),
							new PointF(r.Right, r.Bottom),
							new PointF(r.Left + r.Width / 2 + w / 2, r.Bottom),
							new PointF(r.Left + r.Width / 2 + w / 2, r.Bottom - h),
							new	PointF(r.Left + r.Width / 2 - w / 2, r.Bottom - h),
							new PointF(r.Left + r.Width / 2 - w / 2, r.Bottom),
							new PointF(r.Left, r.Bottom)
						};
					}
					else if (a == (ImageAlignment.Left | ImageAlignment.Bottom))
					{
						// Left-bottom
						contentsPoly = new PointF[]
						{
							new PointF(r.Left, r.Top),
							new PointF(r.Right, r.Top),
							new PointF(r.Right, r.Bottom),
							new PointF(r.Left + w, r.Bottom),
							new PointF(r.Left + w, r.Bottom - h),
							new PointF(r.Left, r.Bottom - h)
						};
					}
					else if (a == ImageAlignment.Left)
					{
						// Left
						contentsPoly = new PointF[]
						{
							new PointF(r.Left, r.Top),
							new PointF(r.Right, r.Top),
							new PointF(r.Right, r.Bottom),
							new PointF(r.Left, r.Bottom),
							new PointF(r.Left, r.Top + r.Height / 2 + h / 2),
							new PointF(r.Left + w, r.Top + r.Height / 2 + h / 2),
							new PointF(r.Left + w, r.Top + r.Height / 2 - h / 2),
							new PointF(r.Left, r.Top + r.Height / 2 - h / 2)
						};
					}
				}
				else
				{
					// Use all available space
					contentsPoly = new PointF[]
					{
						new PointF(r.Left, r.Top),
						new PointF(r.Right, r.Top),
						new	PointF(r.Right, r.Bottom),
						new PointF(r.Left, r.Bottom)
					};
				}

				// Restore clip
				g.Clip = oldClip2;
				newClip2.Dispose();

				// Draw contents text
				if (contentsPoly != null &&
					appointment.DescriptionText != "" &&
					appointment.DescriptionText != null)
				{
					// Draw the text
					MindFusion.Text.StyledText text = new MindFusion.Text.StyledText();
					text.Setup(appointment.DescriptionText, g, context.Style.Font);

					MindFusion.Text.LayoutOptions options = new MindFusion.Text.LayoutOptions();
					MindFusion.Text.Layout layout = new MindFusion.Text.Layout();
					layout.LayoutInPolygon(text, contentsPoly, options, g, context.Style.Font);

					Brush brush = new SolidBrush(context.Style.TextColor);
					layout.Draw(0, 0, new MindFusion.Text.RenderTextCallback(this.DrawText),
						new MindFusion.Text.DrawTextHint(g, context.Style.Font, brush,
						StringFormat.GenericDefault, true));
					brush.Dispose();
				}

				Pen pen = new Pen(context.Style.LineColor, 0);
				g.DrawLine(pen, contentsBounds.Left, contentsBounds.Top,
					contentsBounds.Right, contentsBounds.Top);
				pen.Dispose();
			}

			// Restore clipping
			g.Clip = oldClip;
			newClip.Dispose();

			Pen leftPen = new Pen(context.Style.BorderLeftColor, 0);
			GraphicsPath left = new GraphicsPath();
			AddLeft(bounds.X, bounds.Y, bounds.Width, bounds.Height, radius,
				leftTop, rightTop, leftBottom, rightBottom, left);
			g.DrawPath(leftPen, left);
			left.Dispose();
			leftPen.Dispose();

			Pen topPen = new Pen(context.Style.BorderTopColor, 0);
			GraphicsPath top = new GraphicsPath();
			AddTop(bounds.X, bounds.Y, bounds.Width, bounds.Height, radius,
				leftTop, rightTop, leftBottom, rightBottom, top);
			g.DrawPath(topPen, top);
			top.Dispose();
			topPen.Dispose();

			Pen rightPen = new Pen(context.Style.BorderRightColor, 0);
			GraphicsPath right = new GraphicsPath();
			AddRight(bounds.X, bounds.Y, bounds.Width, bounds.Height, radius,
				leftTop, rightTop, leftBottom, rightBottom, right);
			g.DrawPath(rightPen, right);
			right.Dispose();
			rightPen.Dispose();

			Pen bottomPen = new Pen(context.Style.BorderBottomColor, 0);
			GraphicsPath bottom = new GraphicsPath();
			AddBottom(bounds.X, bounds.Y, bounds.Width, bounds.Height, radius,
				leftTop, rightTop, leftBottom, rightBottom, bottom);
			g.DrawPath(bottomPen, bottom);
			bottom.Dispose();
			bottomPen.Dispose();

			// Dispose the path
			path.Dispose();

			// Restore graphics state
			g.Restore(state);

			// Render the off-screen bitmap to the actual target graphics
			// with the desired transparency
			ImageAttributes attributes = new ImageAttributes();
			ColorMatrix matrix = new ColorMatrix();
			matrix.Matrix33 = (float)appointment.Transparency / 100;
			attributes.SetColorMatrix(matrix);

			// Render the off-screen image
			trueGraphics.DrawImage(obitmap, new Rectangle(context.Bounds.X,
				context.Bounds.Y, obitmap.Width, obitmap.Height), 0, 0,
				obitmap.Width, obitmap.Height, GraphicsUnit.Pixel, attributes);

			// Dispose off-screen objects
			attributes.Dispose();
			ographics.Dispose();
			obitmap.Dispose();

			// Restore the context graphics
			context.Graphics = trueGraphics;
		}

		private void DrawText(string text, RectangleF rect, MindFusion.Text.DrawTextHint hint)
		{
			hint.Graphics.DrawString(text, hint.Font, hint.Brush, rect.Location);
		}

		private static void AddTop(float x, float y, float width, float height, float r,
			bool leftTop, bool rightTop, bool leftBottom, bool rightBottom, GraphicsPath gp)
		{
			float diameter = Math.Min(2 * r, Math.Min(width, height));
			r = diameter / 2;

			if (diameter < width)
				gp.AddLine(x + r, y, x + width - r, y);

			if (r > 0)
			{
				if (rightTop)
				{
					gp.AddArc(x + width - diameter, y, diameter, diameter, 270, 90);
				}
				else
				{
					gp.AddLine(x + r, y, x + width, y);
					gp.AddLine(x + width, y, x + width, y + r);
				}
			}
		}

		private static void AddRight(float x, float y, float width, float height, float r,
			bool leftTop, bool rightTop, bool leftBottom, bool rightBottom, GraphicsPath gp)
		{
			float diameter = Math.Min(2 * r, Math.Min(width, height));
			r = diameter / 2;

			if (diameter < height)
				gp.AddLine(x + width, y + r, x + width, y + height - r);

			if (r > 0)
			{
				if (rightBottom)
				{
					gp.AddArc(x + width - diameter, y + height - diameter, diameter, diameter, 0, 90);
				}
				else
				{
					gp.AddLine(x + width, y + height - r, x + width, y + height);
					gp.AddLine(x + width, y + height, x + width - r, y + height);
				}
			}
		}

		private static void AddBottom(float x, float y, float width, float height, float r,
			bool leftTop, bool rightTop, bool leftBottom, bool rightBottom, GraphicsPath gp)
		{
			float diameter = Math.Min(2 * r, Math.Min(width, height));
			r = diameter / 2;

			if (diameter < width)
				gp.AddLine(x + width - r, y + height, x + r, y + height);

			if (r > 0)
			{
				if (leftBottom)
				{
					gp.AddArc(x, y + height - diameter, diameter, diameter, 90, 90);
				}
				else
				{
					gp.AddLine(x + r, y + height, x, y + height);
					gp.AddLine(x, y + height, x, y + height - r);
				}
			}
		}

		private static void AddLeft(float x, float y, float width, float height, float r,
			bool leftTop, bool rightTop, bool leftBottom, bool rightBottom, GraphicsPath gp)
		{
			float diameter = Math.Min(2 * r, Math.Min(width, height));
			r = diameter / 2;

			if (diameter < height)
				gp.AddLine(x, y + height - r, x, y + r);

			if (r > 0)
			{
				if (leftTop)
				{
					gp.AddArc(x, y, diameter, diameter, 180, 90);
				}
				else
				{
					gp.AddLine(x, y + r, x, y);
					gp.AddLine(x, y, x + r, y);
				}
			}
		}

		/// <summary>
		/// Creates a graphics path of a rounded rectangle with
		/// the specified dimensions and round radius.
		/// </summary>
		private static GraphicsPath CreateRoundRect(
			float x, float y, float width, float height, float r,
			bool leftTop, bool rightTop, bool leftBottom, bool rightBottom)
		{
			GraphicsPath gp = new GraphicsPath();

			try
			{
				AddTop(x, y, width, height, r, leftTop, rightTop, leftBottom, rightBottom, gp);
				AddRight(x, y, width, height, r, leftTop, rightTop, leftBottom, rightBottom, gp);
				AddBottom(x, y, width, height, r, leftTop, rightTop, leftBottom, rightBottom, gp);
				AddLeft(x, y, width, height, r, leftTop, rightTop, leftBottom, rightBottom, gp);

				gp.CloseFigure();
			}
			catch (Exception)
			{
				gp.AddRectangle(new RectangleF(x, y, width, height));
			}
			return gp;
		}
	}
}
